@@ -1,11 +1,12 @@
 import shutil
-from pathlib import Path
 
 import pytest
+from unearth import Link
 
 from pdm.cli import actions
-from pdm.models.pip_shims import Link
+from pdm.exceptions import PdmUsageError
 from pdm.models.specifiers import PySpecSet
+from pdm.utils import path_to_url
 from tests import FIXTURES
 
 
@@ -55,27 +56,23 @@ def test_add_package_to_custom_dev_group(project, working_set):
 
 
 @pytest.mark.usefixtures("repository", "vcs")
-def test_add_editable_package(project, working_set, is_dev):
+def test_add_editable_package(project, working_set):
     # Ensure that correct python version is used.
     project.environment.python_requires = PySpecSet(">=3.6")
-    actions.do_add(project, is_dev, packages=["demo"])
+    actions.do_add(project, True, packages=["demo"])
     actions.do_add(
         project,
-        is_dev,
+        True,
         editables=["git+https://github.com/test-root/demo.git#egg=demo"],
     )
-    group = (
-        project.tool_settings["dev-dependencies"]["dev"]
-        if is_dev
-        else project.meta["dependencies"]
-    )
-    assert "demo" in group[0]
-    assert "-e git+https://github.com/test-root/demo.git#egg=demo" in group[1]
+    group = project.tool_settings["dev-dependencies"]["dev"]
+    assert group == ["-e git+https://github.com/test-root/demo.git#egg=demo"]
     locked_candidates = project.locked_repository.all_candidates
     assert (
         locked_candidates["demo"].prepare(project.environment).revision
         == "1234567890abcdef"
     )
+    assert working_set["demo"].link_file
     assert locked_candidates["idna"].version == "2.7"
     assert "idna" in working_set
 
@@ -83,36 +80,37 @@ def test_add_editable_package(project, working_set, is_dev):
     assert not working_set["demo"].link_file
 
 
-@pytest.mark.usefixtures("repository", "vcs")
-def test_editable_package_override_non_editable(project, working_set):
+@pytest.mark.usefixtures("repository", "vcs", "working_set")
+def test_add_editable_package_to_metadata_forbidden(project):
     project.environment.python_requires = PySpecSet(">=3.6")
-    actions.do_add(
-        project, packages=["git+https://github.com/test-root/demo.git#egg=demo"]
-    )
-    actions.do_add(
-        project,
-        editables=["git+https://github.com/test-root/demo.git#egg=demo"],
-    )
-    assert working_set["demo"].link_file
+    with pytest.raises(PdmUsageError):
+        actions.do_add(
+            project, editables=["git+https://github.com/test-root/demo.git#egg=demo"]
+        )
+    with pytest.raises(PdmUsageError):
+        actions.do_add(
+            project,
+            group="foo",
+            editables=["git+https://github.com/test-root/demo.git#egg=demo"],
+        )
 
 
 @pytest.mark.usefixtures("repository", "vcs")
-def test_non_editable_no_override_editable(project, working_set, is_editable):
+def test_non_editable_override_editable(project, working_set):
     project.environment.python_requires = PySpecSet(">=3.6")
     actions.do_add(
         project,
+        dev=True,
         editables=[
             "git+https://github.com/test-root/demo.git#egg=demo",
-            "git+https://github.com/test-root/demo-module.git#egg=demo-module",
         ],
     )
     actions.do_add(
         project,
+        dev=True,
         packages=["git+https://github.com/test-root/demo.git#egg=demo"],
-        no_editable=not is_editable,
     )
-    assert working_set["demo-module"].link_file
-    assert bool(working_set["demo"].link_file) is is_editable
+    assert not project.dev_dependencies["demo"].editable
 
 
 @pytest.mark.usefixtures("repository", "working_set")
@@ -268,11 +266,13 @@ def test_add_cached_vcs_requirement(project, mocker):
     url = "git+https://github.com/test-root/demo.git@1234567890abcdef#egg=demo"
     built_path = FIXTURES / "artifacts/demo-0.0.1-py2.py3-none-any.whl"
     wheel_cache = project.make_wheel_cache()
-    cache_path = Path(wheel_cache.get_path_for_link(Link(url)))
+    cache_path = wheel_cache.get_path_for_link(
+        Link(url), project.environment.target_python
+    )
     if not cache_path.exists():
         cache_path.mkdir(parents=True)
     shutil.copy2(built_path, cache_path)
-    downloader = mocker.patch("pdm.models.pip_shims.unpack_url")
+    downloader = mocker.patch("unearth.finder.unpack_link")
     builder = mocker.patch("pdm.builders.WheelBuilder.build")
     actions.do_add(project, packages=[url], no_self=True)
     lockfile_entry = next(p for p in project.lockfile["package"] if p["name"] == "demo")
@@ -295,3 +295,22 @@ def test_add_with_prerelease(project, working_set):
     actions.do_add(project, packages=["urllib3"], prerelease=True)
     assert working_set["urllib3"].version == "1.23b0"
     assert project.meta.dependencies[0] == "urllib3<2,>=1.23b0"
+
+
+@pytest.mark.usefixtures("repository")
+def test_add_editable_package_with_extras(project, working_set):
+    project.environment.python_requires = PySpecSet(">=3.6")
+    dep_path = FIXTURES.joinpath("projects/demo").as_posix()
+    actions.do_add(
+        project,
+        dev=True,
+        group="dev",
+        editables=[f"{dep_path}[security]"],
+    )
+    assert (
+        f"-e {path_to_url(dep_path)}#egg=demo[security]"
+        in project.get_pyproject_dependencies("dev", True)
+    )
+    assert "demo" in working_set
+    assert "requests" in working_set
+    assert "urllib3" in working_set
