@@ -2,8 +2,8 @@ import os
 
 import pytest
 
-from pdm.models.pip_shims import path_to_url
 from pdm.models.requirements import RequirementError, parse_requirement
+from pdm.utils import path_to_url
 from tests import FIXTURES
 
 FILE_PREFIX = "file:///" if os.name == "nt" else "file://"
@@ -21,7 +21,7 @@ REQUIREMENTS = [
     ),
     (
         'pip @ https://github.com/pypa/pip/archive/1.3.1.zip ; python_version > "3.4"',
-        'pip @ https://github.com/pypa/pip/archive/1.3.1.zip; python_version > "3.4"',
+        None,
     ),
     (
         "git+http://git.example.com/MyProject.git@master#egg=MyProject",
@@ -33,16 +33,15 @@ REQUIREMENTS = [
     ),
     (
         (FIXTURES / "projects/demo").as_posix(),
-        "demo @ " + path_to_url((FIXTURES / "projects/demo")),
+        "demo @ " + path_to_url(FIXTURES / "projects/demo"),
     ),
     (
         (FIXTURES / "artifacts/demo-0.0.1-py2.py3-none-any.whl").as_posix(),
-        "demo @ "
-        + path_to_url((FIXTURES / "artifacts/demo-0.0.1-py2.py3-none-any.whl")),
+        "demo @ " + path_to_url(FIXTURES / "artifacts/demo-0.0.1-py2.py3-none-any.whl"),
     ),
     (
         (FIXTURES / "projects/demo").as_posix() + "[security]",
-        "demo[security] @ " + path_to_url((FIXTURES / "projects/demo")),
+        "demo[security] @ " + path_to_url(FIXTURES / "projects/demo"),
     ),
     (
         'requests; python_version=="3.7.*"',
@@ -62,7 +61,6 @@ REQUIREMENTS = [
 @pytest.mark.parametrize("req, result", REQUIREMENTS)
 def test_convert_req_dict_to_req_line(req, result):
     r = parse_requirement(req)
-    assert r.as_ireq()
     result = result or req
     assert r.as_line() == result
 
